/* graph.cpp - Morse Code Dojo graph widget
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include <qpainter.h>
#include <stdio.h>

#include "graph.h"


Graph::Graph( QWidget *parent = NULL, const char *name = NULL ) 
   : QWidget( parent, name )
{
   setBackgroundColor( QColor( 255, 255, 255 ) );
   m_graphColor.setRgb( 0, 0, 255 );

   reset();
}

Graph::~Graph()
{
}

void Graph::addPoint( int x, int y )
{
   m_list[x] = y;

   QPainter p;
   p.begin(this);
   //p.setPen( QPainter::NoPen );
   p.setPen( QPen( QColor(255, 255, 255), 2, QPen::SolidLine ) );

   float percent = (float) this->width() / (float) MAX_GRID_SIZE;
   int x1 = (int) ( (float) percent * x);
   int y1 = (int) (((float) y / (float) m_max) * this->height());

   p.setBrush( QBrush( QColor(255, 255, 255), QBrush::SolidPattern ) );
   p.drawRect( x1, 0, (int) ( (float) this->width() / (float) (MAX_GRID_SIZE) ), this->height());

   p.setPen( QPen( m_graphColor, 2, QPen::SolidLine ) );
   p.setBrush( QBrush( m_graphColor, QBrush::SolidPattern ) );
   p.drawRect( x1, this->height() - y1, (int) ( (float) this->width() / (float) (MAX_GRID_SIZE) ), y1);

   p.end();
}

void Graph::setScale(int min, int max)
{
   m_min = min;
   m_max = max;
}

void Graph::setScale(int max)
{
   m_min = 0;
   m_max = max;
}

void Graph::paintEvent( QPaintEvent *e )
{
   QPainter p;
   p.begin(this);
   p.setPen( QPen( m_graphColor, 2, QPen::SolidLine ) );
   p.setBrush( QBrush( m_graphColor, QBrush::SolidPattern ) );

   for ( int t = 0; t < MAX_GRID_SIZE; t++)
   {
      float percent = (float) this->width() / (float) MAX_GRID_SIZE;
      int x = (int) ( (float) percent * t);
      int y = (int) (((float) m_list[t] / (float) m_max) * this->height());
      p.drawRect( x, this->height() - y, (int) ( (float) this->width() / (float) (MAX_GRID_SIZE) ), y);
   }
   p.end();
}

void Graph::mousePressEvent( QMouseEvent *e )
{
   float x1, y1;

   if( m_clickable && e->button() == LeftButton )
   {
      x1 = e->x();
      y1 = e->y();
      
      y1 = ( (this->height() - y1) / this->height() );
      y1 = y1 * m_max + 0.5 ;
      x1 = (x1 / this->width()) * MAX_GRID_SIZE;

      int x = (int) x1;
      int y = (int) y1;

      if ( m_list[x] != y )
      {
         emit clickedAt( x, y );
      }
   }
   else
   {
      QWidget::mousePressEvent( e );
   }
}

void Graph::mouseMoveEvent( QMouseEvent *e )
{
   float x1, y1;

   if( m_clickable && e->state() == LeftButton 
         && (e->x() >= 0) && (e->x() <= this->width() ) )
   {
      x1 = e->x();
      y1 = e->y();

      if ( y1 < 0 )
        y1 = 0;

      if ( y1 > this->height() )
         y1 = this->height();
      
      y1 = ( (this->height() - y1) / this->height() );
      y1 = y1 * m_max + 0.5 ;
      x1 = (x1 / this->width()) * MAX_GRID_SIZE;

      int x = (int) x1;
      int y = (int) y1;

      if ( m_list[x] != y )
      {
         emit clickedAt( x, y );
      }
   }
   else
   {
      QWidget::mouseMoveEvent( e );
   }
}

void Graph::reset(void)
{
   m_min =   0;
   m_max = 100;
   m_clickable = false;

   for ( int t = 0; t < MAX_GRID_SIZE; t++ )
      m_list[t] = 0;
}

void Graph::showGraph( void )
{
   this->update();
}

void Graph::setClickable(bool o)
{
   m_clickable = o;
}

bool Graph::isClickable(void)
{
   return m_clickable;
}
