/* lesson.cpp - Morse Code Dojo lesson widget
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtimer.h>
#include <ctype.h>
#include <unistd.h>

#include "lesson.h"
#include "morse.h"
#include "mrand.h"

/* 
 *  Constructs a Lesson which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
Lesson::Lesson( QWidget* parent,  const char* name, bool modal, WFlags fl )
   : QDialog( parent, name, modal, fl )
{
   if ( !name )
   setName( "Lesson" );
   resize( 575, 450 ); 
   setCaption( tr( "Lesson" ) );
   LessonLayout = new QVBoxLayout( this ); 
   LessonLayout->setSpacing( 6 );
   LessonLayout->setMargin( 11 );

   Frame4 = new QFrame( this, "Frame4" );
   Frame4->setFrameShape( QFrame::Box );
   Frame4->setFrameShadow( QFrame::Sunken );
   Frame4Layout = new QVBoxLayout( Frame4 ); 
   Frame4Layout->setSpacing( 6 );
   Frame4Layout->setMargin( 11 );
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
   Frame4Layout->addItem( spacer );

   sent_v = new QLabel( Frame4, "sent_v" );
   sent_v->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, sent_v->sizePolicy().hasHeightForWidth() ) );
   QFont sent_v_font(  sent_v->font() );
   sent_v_font.setPointSize( 20 );
   sent_v->setFont( sent_v_font ); 
   sent_v->setText( "" );
   sent_v->setAlignment( int( QLabel::AlignCenter ) );
   Frame4Layout->addWidget( sent_v );

   message_v = new QLabel( Frame4, "message_v" );
   QFont message_v_font(  message_v->font() );
   message_v_font.setPointSize( 20 );
   message_v->setFont( message_v_font ); 
   message_v->setText( "" );
   message_v->setAlignment( int( QLabel::AlignCenter ) );
   Frame4Layout->addWidget( message_v );

   QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
   Frame4Layout->addItem( spacer_2 );
   LessonLayout->addWidget( Frame4 );

   Layout8 = new QHBoxLayout; 
   Layout8->setSpacing( 6 );
   Layout8->setMargin( 0 );

   start_b = new QPushButton( this, "start_b" );
   start_b->setText( tr( "Start" ) );
   Layout8->addWidget( start_b );

   stop_b = new QPushButton( this, "stop_b" );
   stop_b->setText( tr( "Stop" ) );
   Layout8->addWidget( stop_b );

   close_b = new QPushButton( this, "close_b" );
   close_b->setText( tr( "Close" ) );
   Layout8->addWidget( close_b );
   LessonLayout->addLayout( Layout8 );

   resend_t  = new QTimer(this, "");
   elapsed_t = new QTimer(this, "");

   // signals and slots connections
   connect( close_b,   SIGNAL( clicked() ), this, SLOT( close() ) );
   connect( start_b,   SIGNAL( clicked() ), this, SLOT( startPressed() ) );
   connect( stop_b,    SIGNAL( clicked() ), this, SLOT( stopPressed() ) );
   connect( resend_t,  SIGNAL( timeout() ), this, SLOT( resendTimeout() ) );
   connect( elapsed_t, SIGNAL( timeout() ), this, SLOT( elapsedTimeout() ) );

   m_morse = Morse::getInstance();
   m_maxTime   = 60;
   m_mRand = MRand::getInstance();

   m_fp            = NULL;
   m_timeRemaining = 0;
   m_state         = LS_None;
}

Lesson::~Lesson()
{
   delete m_morse;
}

void Lesson::closeEvent(QCloseEvent *e)
{
   m_morse->abort();
   resend_t->stop();
   elapsed_t->stop();

   if ( m_fp )
   {
      fclose(m_fp);
      m_fp = NULL;
   }

   emit closed();
   QDialog::closeEvent(e);
}

void Lesson::show()
{
   m_morse->abort();
   elapsed_t->stop();
   resend_t->stop();

   message_v->setFocus();

   m_current   = 0;
   m_lastWrong = 0;
   m_sent      = "";
   m_max       = -1;
   
   sent_v->setText("<html>");
   message_v->setText("<html>");

   QDialog::show();
}

/*  
 *  Main event handler. Reimplemented to handle application
 *  font changes
 */
bool Lesson::event( QEvent* ev )
{
   bool ret = QDialog::event( ev ); 
   if ( ev->type() == QEvent::ApplicationFontChange ) {
      QFont sent_v_font(  sent_v->font() );
      sent_v_font.setPointSize( 20 );
      sent_v->setFont( sent_v_font ); 
      QFont message_v_font(  message_v->font() );
      message_v_font.setPointSize( 20 );
      message_v->setFont( message_v_font ); 
   }
   return ret;
}

void Lesson::keyPressEvent(QKeyEvent *e)
{
   char ch = e->ascii();
   ch = toupper(ch);
   if ( (ch >= ' ' && ch <='~') )
   {
      QString tempNum;
      message_v->setText( tr( message_v->text().append( QChar(ch)) ) );
   }
   else if( e->key() == Key_Enter || e->key() == Key_Return )
   {
      message_v->setText("<html>");
   }
   else if( e->key() == Key_Escape )
   {
      m_timeRemaining = 0;
      m_current = 0;
      m_morse->abort();
   }
   else
   {
      QDialog::keyPressEvent(e);
   }
}

void Lesson::stopPressed(void)
{
   m_morse->abort();
   elapsed_t->stop();
   resend_t->stop();
}

void Lesson::startPressed(void)
{
   m_morse->abort();
   elapsed_t->stop();
   resend_t->stop();

   message_v->setFocus();

   if ( m_fp )
   {
      fclose(m_fp);
      m_fp = NULL;
   }
   
   m_fp = fopen ( "lessons/lesson01.dat", "r");
   if ( m_fp == NULL )
   {
      fprintf(stderr, "Could not open file\n");
      return;
   }

   m_current   = 0;
   m_lastWrong = 0;
   m_sent      = "";
   m_max       = -1;
   
   sent_v->setText("<html>");
   message_v->setText("<html>");

   elapsed_t->start(250, FALSE);

   doLine();
}

void Lesson::resendTimeout(void)
{
   m_max       = -1;
   
   message_v->setText("<html>");

   m_morse->send( (char) (QChar) m_sent[m_current] );
}

void Lesson::elapsedTimeout(void)
{
   printf("timeout: %d\n", m_timeRemaining);
   switch ( m_state )
   {
      case LS_None:
         break;
      case LS_End:
         elapsed_t->stop();
         m_state = LS_None;
         break;
      case LS_Pause:
         m_timeRemaining--;
         if ( m_timeRemaining < 0 )
            doLine();
         break;
      case LS_Teach:
         if ( ! m_morse->sending() )
         {
            m_timeRemaining--;
            if ( m_timeRemaining > 0)
               m_morse->send(m_sent);
            else
               doLine();
         }
         break;
      case LS_Random:
         if ( ! m_morse->sending() )
         {
            m_current--;
            if ( m_current < 0 )
            {
               m_timeRemaining--;
               
               if ( m_timeRemaining > 0 )
               {
                  int letters = 1;
                  while ( letters <= 1 )
                  {
                     char *temp = m_mRand->getString();
                     m_sent = temp;
                     for ( unsigned t=0; t<strlen(temp); t++)
                     {
                        if ( t > 0 && temp[t] != temp[t-1] )
                        {
                           letters++;
                        }
                     }
                     free(temp);
                  }

                  m_sent += " ";

                  sent_v->setText("<html>");
                  message_v->setText("<html>");
                  m_morse->send(m_sent);
                  m_current = 2;
               }
               else
               {
                  doLine();
               }
            }
            else
            {
               if ( m_current == 0 )
               {
                  sent_v->setText(m_sent);
               }

               m_morse->send(m_sent);
            }
         }
         break;
      default:
         fprintf(stderr, "Lesson::elapsedTimeout() state switch fell through\n");
         elapsed_t->stop();
         break;
   }
}

void Lesson::doLine()
{
   char input[1024];
   int letters;

   if ( m_fp )
   {
      while ( fgets(input, sizeof(input) - 1, m_fp) )
      {
         printf("%s", input);
         switch ( toupper(input[0]) )
         {
            case 'M':
               message_v->setText("<html>");
               sent_v->setText( &input[1] );
               break;
            case 'P':
               m_timeRemaining = atoi(&input[1]);
               m_state = LS_Pause;
               return;
               break;
            case 'T':
               message_v->setText("<html>");
               m_state = LS_Teach;
               sent_v->setText(QChar(toupper(input[1])));
               m_sent.sprintf("%c ", input[1]);
               m_morse->send(m_sent);
               m_timeRemaining = 10;
               return;
            case 'R':
               message_v->setText("<html>");
               m_state = LS_Random;

               m_mRand->clearWeight();
               for ( unsigned t=1; t<strlen(input); t++)
               {
                  m_mRand->setCharWeight( toupper(input[t]), 1);
               }

               m_mRand->setWordRange(3, 3);

               letters = 1;
               while ( letters <= 1 )
               {
                  char *temp = m_mRand->getString();
                  m_sent = temp;
                  for ( unsigned t=0; t<strlen(temp); t++)
                  {
                     if ( t > 0 && temp[t] != temp[t-1] )
                     {
                        letters++;
                     }
                  }
                  free(temp);
               }

               m_sent += " ";

               sent_v->setText("<html>");
               m_morse->send(m_sent);
               m_timeRemaining = 4;
               m_current = 2;
               return;
               break;
            case 'W':
               message_v->setText("<html>");
               break;
            default:
               fprintf(stderr, "Lesson::doLine() switch fell through\n");
               elapsed_t->stop();
               break;
         }
      }

      m_state = LS_End;

      fclose( m_fp );
      m_fp = NULL;
   }
}

void Lesson::setLesson(int l)
{
   m_lesson = l;
   return;
}

