/* mainwin.cpp - Morse Code Dojo main (menu) window
 *  
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include "mainwin.h"

#include <qapplication.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "morse.h"
#include "recgame.h"
#include "lesson.h"
#include "practice.h"
#include "bprop.h"
#include "rprop.h"
#include "weightwin.h"
#include "prof.h"

/* 
 *  Constructs a MainWin which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
MainWin::MainWin( QWidget* parent,  const char* name, bool modal, WFlags fl )
   : QDialog( parent, name, modal, fl )
{
   if ( !name )
   setName( "MainWin" );
   resize( 348, 217 ); 
   setCaption( tr( "Morse Code Dojo" ) );
   MainWinLayout = new QVBoxLayout( this ); 
   MainWinLayout->setSpacing( 6 );
   MainWinLayout->setMargin( 11 );

   Layout6 = new QHBoxLayout; 
   Layout6->setSpacing( 6 );
   Layout6->setMargin( 0 );

   GroupBox1 = new QGroupBox( this, "GroupBox1" );
   GroupBox1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, GroupBox1->sizePolicy().hasHeightForWidth() ) );
   GroupBox1->setTitle( tr( "Learn" ) );
   GroupBox1->setColumnLayout(0, Qt::Vertical );
   GroupBox1->layout()->setSpacing( 0 );
   GroupBox1->layout()->setMargin( 0 );
   GroupBox1Layout = new QVBoxLayout( GroupBox1->layout() );
   GroupBox1Layout->setAlignment( Qt::AlignTop );
   GroupBox1Layout->setSpacing( 6 );
   GroupBox1Layout->setMargin( 11 );

   lesson_b = new QPushButton( GroupBox1, "lesson_b" );
   lesson_b->setText( tr( "Lessons" ) );
   GroupBox1Layout->addWidget( lesson_b );

   koch_b = new QPushButton( GroupBox1, "koch_b" );
   koch_b->setText( tr( "Koch Method" ) );
   GroupBox1Layout->addWidget( koch_b );

   practice_b = new QPushButton( GroupBox1, "practice_b" );
   practice_b->setText( tr( "Practice" ) );
   GroupBox1Layout->addWidget( practice_b );
   Layout6->addWidget( GroupBox1 );

   GroupBox2 = new QGroupBox( this, "GroupBox2" );
   GroupBox2->setTitle( tr( "Exercises" ) );
   GroupBox2->setColumnLayout(0, Qt::Vertical );
   GroupBox2->layout()->setSpacing( 0 );
   GroupBox2->layout()->setMargin( 0 );
   GroupBox2Layout = new QVBoxLayout( GroupBox2->layout() );
   GroupBox2Layout->setAlignment( Qt::AlignTop );
   GroupBox2Layout->setSpacing( 6 );
   GroupBox2Layout->setMargin( 11 );

   recgame_b = new QPushButton( GroupBox2, "recgame_b" );
   recgame_b->setText( tr( "Receiving Game" ) );
   GroupBox2Layout->addWidget( recgame_b );

   prof_b = new QPushButton( GroupBox2, "prof_b" );
   prof_b->setText( tr( "Proficiency" ) );
   GroupBox2Layout->addWidget( prof_b );
   Layout6->addWidget( GroupBox2 );
   MainWinLayout->addLayout( Layout6 );

   Layout5 = new QHBoxLayout; 
   Layout5->setSpacing( 6 );
   Layout5->setMargin( 0 );
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout5->addItem( spacer );

   weight_b = new QPushButton( this, "weight_b" );
   weight_b->setText( tr( "Weight" ) );
   Layout5->addWidget( weight_b );

   ok_b = new QPushButton( this, "ok_b" );
   ok_b->setText( tr( "Quit" ) );
   Layout5->addWidget( ok_b );
   MainWinLayout->addLayout( Layout5 );

   // signals and slots connections
   connect( lesson_b, SIGNAL( clicked() ), this, SLOT( lessonsClicked() ) );
   connect( koch_b, SIGNAL( clicked() ), this, SLOT( kochClicked() ) );
   connect( practice_b, SIGNAL( clicked() ), this, SLOT( practiceClicked() ) );
   connect( prof_b, SIGNAL( clicked() ), this, SLOT( profClicked() ) );
   connect( recgame_b, SIGNAL( clicked() ), this, SLOT( recGameClicked() ) );
   connect( weight_b, SIGNAL( clicked() ), this, SLOT( weightClicked() ) );
   connect( ok_b, SIGNAL( clicked() ), this, SLOT( okClicked() ) );

   m_morse     = Morse::getInstance();
   m_mRand     = MRand::getInstance();
   m_morse->setSpeed(17, 8);

   m_weight    = new WeightWin();
   m_recGame   = new RecGame();
   m_practice  = new Practice();
   m_lesson    = new Lesson();
   m_prof      = new Prof();

   connect( m_lesson, SIGNAL( closed() ),   this, SLOT( show() ) );
//   connect( m_koch, SIGNAL( closed() ),     this, SLOT( show() ) );
   connect( m_practice, SIGNAL( closed() ), this, SLOT( show() ) );
   connect( m_prof, SIGNAL( closed() ),     this, SLOT( show() ) );
   connect( m_recGame, SIGNAL( closed() ),  this, SLOT( show() ) );
   connect( m_weight, SIGNAL( closed() ),   this, SLOT( show() ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
MainWin::~MainWin()
{
   // no need to delete child widgets, Qt does it all for us
}

void MainWin::kochClicked()
{
   qWarning( "MainWin::kochClicked(): Not implemented yet!" );
}

void MainWin::lessonsClicked()
{
   BasicProperties b;

   b.setCharSpeed(m_morse->characterSpeed());
   b.setWordSpeed(m_morse->wordSpeed());


   if ( b.exec() ) 
   {
      m_morse->setSpeed(b.getCharSpeed(), b.getWordSpeed() );

      this->hide();
      m_lesson->show();
   }
}

void MainWin::okClicked()
{
   m_morse->abort();
   qApp->quit();
}

void MainWin::practiceClicked()
{
   BasicProperties b;

   b.setCharSpeed(m_morse->characterSpeed());
   b.setWordSpeed(m_morse->wordSpeed());


   if ( b.exec() ) 
   {
      m_morse->setSpeed(b.getCharSpeed(), b.getWordSpeed() );
      m_practice->setTime( b.getTime() );

      this->hide();
      m_practice->show();
   }
}

void MainWin::profClicked()
{
   BasicProperties b;

   b.setCharSpeed(m_morse->characterSpeed());
   b.setWordSpeed(m_morse->wordSpeed());

   if ( b.exec() ) 
   {
      m_morse->setSpeed(b.getCharSpeed(), b.getWordSpeed() );

      this->hide();
      m_prof->show();
   }
}

void MainWin::recGameClicked()
{
   RecProperties r;

   r.setCharSpeed(m_morse->characterSpeed());
   r.setWordSpeed(m_morse->wordSpeed());

   r.setMinLength( m_mRand->minLength() );
   r.setMaxLength( m_mRand->maxLength() );

   if ( r.exec() ) 
   {
      m_morse->setSpeed(r.getCharSpeed(), r.getWordSpeed() );
      m_recGame->setTime( r.getTime() );

      m_mRand->setWordRange( r.getMinLength(), r.getMaxLength() );

      this->hide();
      m_recGame->show();
   }
}

void MainWin::weightClicked()
{
   this->hide();
   m_weight->show();
}
