/* morse.cpp - Morse Code Dojo morse code object
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <linux/soundcard.h>
#include <sys/ioctl.h>
#include <termios.h>

#include "morse.h"

Morse * Morse::s_instance = NULL;

Morse::Morse(unsigned char_speed = 17, unsigned word_speed = 8, unsigned char_tone = 720) : QWidget(NULL, "")
{
   m_sendQueue[0]  = '\0';
   m_queueLocation = 0;

   setCharacterSpeed(char_speed);
   setWordSpeed(word_speed);

   setCharacterTone(char_tone);

   for(int t=0; t<MORSE_RATE; t++)
      m_silence[t] = 128;

   for(int t=0; t<MORSE_RATE; t++)
   {
      m_buzz[t] = 128 + (char) 50 * ( sin( (double) t / 7.5) < 0 ? -1 : 1 );
   }

   double PI = atan(1) * 4.0;
   int d;
   for ( unsigned t = 0; t < sizeof(m_buf); t++)
   {
      if( t < 1000 )
      {
         double r = ((signed) t / ((float) MORSE_RATE / 1000.0)) * (2.0 * PI) ;
         m_bell[t] = 128 + (int) ( (float) (sin(r) * 50));
      }
      else if( t < 3500)
      {
         d = t - 1000;
         double r = ((signed) d / ((float) MORSE_RATE / (float) m_tone)) * (2.0 * PI) ;
         if(d < 2500)
            m_bell[t] = 128 + (int) ( (float) (sin(r) * (50 - (d/50))));
         else
            m_bell[t] = 128;
      }
      else
      {
         m_bell[t] = 128;
      }
   }

   send_t = new QTimer(this, "");

   connect( send_t, SIGNAL(timeout()), SLOT(sendTimeout()) );

   openDevice();
}

Morse::~Morse()
{
   fclose(m_fp);
}

Morse * Morse::getInstance()
{
   if ( s_instance == NULL )
   {
      s_instance = new Morse();
   }

   return s_instance;
}

bool Morse::setSpeed(unsigned charspeed, unsigned wordspeed)
{
   if( charspeed < wordspeed)
      return false;

   if ( setCharacterSpeed(charspeed) == true && setWordSpeed(wordspeed) == true )
   {
      return true;
   }
   else
   {
      return false;
   }
}

void Morse::openDevice()
{
   int format = AFMT_U8;
   int channels = 1;
   int speed = MORSE_RATE;
   
   m_fp = fopen("/dev/dsp", "w");
   m_fd = fileno(m_fp);
   ioctl(m_fd, SNDCTL_DSP_SETFMT, &format);
   ioctl(m_fd, SNDCTL_DSP_CHANNELS, &channels);
   ioctl(m_fd, SNDCTL_DSP_SPEED, &speed);

   audio_buf_info info;
   ioctl(m_fd, SNDCTL_DSP_GETOSPACE, &info);
   m_maxBytes = info.bytes;
   
//   printf("\nAudio buffer info:\n");
//   printf("  Free Fragments:  %d\n", info.fragments);
//   printf("  Total Fragments: %d\n", info.fragstotal);
//   printf("  Free Bytes:      %d\n", info.bytes);
}

void Morse::flushDevice()
{
   fflush(m_fp);
}

bool Morse::sending()
{
   audio_buf_info info;
   ioctl(m_fd, SNDCTL_DSP_GETOSPACE, &info);

   if ( m_maxBytes == info.bytes && m_sendQueue[m_queueLocation] == '\0')
      return false;
   else
      return true;
   
}

unsigned Morse::characterSpeed()
{
   return m_charWpm;
}

unsigned Morse::wordSpeed()
{
   return m_wordWpm;
}

bool Morse::setCharacterSpeed(unsigned char_speed)
{
   m_charWpm  = char_speed;
   m_charTick = 60.0 / (50.0 * (double) char_speed);
   m_charDur  = (int) ( (double) m_charTick * MORSE_RATE);
   return(true);
}

bool Morse::setWordSpeed(unsigned word_speed)
{
   m_wordWpm  = word_speed;
   m_wordTick = ( 60 / m_wordWpm - 31.0 * m_charTick) / 19;
   m_wordDur  = (int) ( (double) m_wordTick * MORSE_RATE );

   return(true);
}

bool Morse::setCharacterTone(unsigned tone)
{
   if ( tone < 500 || tone > 1500 )
      return(false);

   m_tone = tone;

   double PI = atan(1) * 4.0;
   for ( unsigned t = 0; t < sizeof(m_buf); t++)
   {
      double r = ((signed) t / ((float) MORSE_RATE / (float) m_tone)) * (2.0 * PI) ;
      m_buf[t] = 128 + (int) ( (float) (sin(r) * 50));
   }

   return(true);
}

void Morse::sendInternal(char ch)
{
   ch = toupper(ch);

   if ( ch <  32 || ch >= 96 )
      return;

   printf("%c = %s\n", ch, morse_char[ch]);

   for ( int t=0; morse_char[ch][t]; t++ )
   {
      switch ( morse_char[ch][t] )
      {
         case '.':
            fwrite(m_buf, m_charDur, 1, m_fp);
            break;
         case '-':
            fwrite(m_buf, m_charDur * 3, 1, m_fp);
            break;
         case ' ':
            fwrite(m_silence, m_wordDur * 2, 1, m_fp);
            fwrite(m_silence, m_wordDur * 2, 1, m_fp);
            fwrite(m_silence, m_wordDur * 2, 1, m_fp);
            fwrite(m_silence, m_wordDur, 1, m_fp);
            break;
         default:
            fprintf(stderr, "No character match!\n");
      }

      if(morse_char[ch][t+1])
         fwrite(m_silence, m_charDur, 1, m_fp);
   }
//   fwrite(m_silence, m_wordDur, 1, m_fp);
}

void Morse::send(char ch)
{
   ch = toupper(ch);

   if ( ch <= 32 || ch >= 96 )
      return;

   if ( m_sendQueue[m_queueLocation] )
   {
      queue ( ch );
      return;
   }

   if ( canSend() != true )
   {
      queue( ch );
   }
   else
   {
      sendInternal(ch);
      flushDevice();
   }
}

void Morse::send(const char *s)
{
   if ( m_sendQueue[m_queueLocation] )
   {
      queue ( s );
      return;
   }

   for ( int t = 0; s[t]; t++ )
   {
      sendInternal(s[t]);
      if(s[t + 1] && s[t + 1] != ' ')
      {
         fwrite(m_silence, m_wordDur * 2, 1, m_fp);
         fwrite(m_silence, m_wordDur, 1, m_fp);
      }

      if ( canSend() == false )
      {
         queue( &s[t+1] );
         break;
      }
   }
   flushDevice();
}

void Morse::sendSpace(const char *s)
{
   for ( int t = 0; s[t]; t++ )
   {
      sendInternal(s[t]);
      if(s[t + 1])
      {
         fwrite(m_silence, m_wordDur * 2, 1, m_fp);
         sendInternal(' ');
      }
   }
   flushDevice();
}

void Morse::abort(void)
{
   ioctl(m_fd, SNDCTL_DSP_RESET, NULL);
   fclose(m_fp);
   openDevice();
}

void Morse::correct()
{
   abort();
   fwrite(m_bell, MORSE_RATE >> 1, 1, m_fp);
}

void Morse::incorrect()
{
   abort();
   fwrite(m_buzz, MORSE_RATE >> 1, 1, m_fp);
}

bool Morse::canSend(void)
{
   audio_buf_info info;
   ioctl(m_fd, SNDCTL_DSP_GETOSPACE, &info);
   
//   if ( info.fragments < 100 )
   if ( info.fragments < 200 )
      return false;
   else
      return true;
}

bool Morse::queue(char ch)
{
   if ( m_queueLocation > (SEND_QUEUE_SIZE >> 1) )
   {
      memmove( m_sendQueue, &m_sendQueue[m_queueLocation], SEND_QUEUE_SIZE - m_queueLocation );
   }

   m_queueLocation = 0;

   int len = strlen ( m_sendQueue );
   if( len + 1 <= SEND_QUEUE_SIZE )
      return false;

   m_sendQueue[len]   = ch;
   m_sendQueue[len+1] = '\0';

   if ( ! send_t->isActive() )
      send_t->start(3000, false);
   return true;
}

bool Morse::queue(const char *s)
{
   if ( m_queueLocation > (SEND_QUEUE_SIZE >> 1) )
   {
      memmove( m_sendQueue, &m_sendQueue[m_queueLocation], SEND_QUEUE_SIZE - m_queueLocation );
   }

   m_queueLocation = 0;

   if ( strlen(m_sendQueue) + strlen(s) >= SEND_QUEUE_SIZE )
      return false;

   strcat(m_sendQueue, s);

   if ( ! send_t->isActive() )
      send_t->start(3000, false);

   return true;
}

void Morse::sendTimeout(void)
{
   while ( canSend() == true && m_sendQueue[m_queueLocation] && m_queueLocation < SEND_QUEUE_SIZE)
   {
      sendInternal(m_sendQueue[m_queueLocation]);

      if(m_sendQueue[m_queueLocation + 1] && m_sendQueue[m_queueLocation] != ' ')
      {
         fwrite(m_silence, m_wordDur * 2, 1, m_fp);
         fwrite(m_silence, m_wordDur, 1, m_fp);
      }

      m_queueLocation++;
   }

   flushDevice();

   /* Stop at the end of the send queue */
   if ( m_queueLocation >= SEND_QUEUE_SIZE )
   {
      m_queueLocation = 0;
      m_sendQueue[m_queueLocation] = '\0';
   }

   /* Reset send queue when finished */
   if ( m_sendQueue[m_queueLocation] == '\0')
   {
      m_queueLocation = 0;
      m_sendQueue[m_queueLocation] = '\0';
      send_t->stop();
   }
}

long Morse::length(char ch)
{
   double n;
   ch = toupper(ch);

   if ( ch == 32 )
   {
      return ( (long) (m_wordTick * 7 * 1000) );
   }
   
   if ( ch <  32 || ch >= 96 )
      return 0;

   n = 0;
   for ( int t=0; morse_char[ch][t]; t++ )
   {
      switch ( morse_char[ch][t] )
      {
         case '.':
            n += m_charTick;
            break;
         case '-':
            n += m_charTick * 3.0;
            break;
         default:
            fprintf(stderr, "No character match!\n");
      }

      if(morse_char[ch][t+1])
         n += m_charTick;
   }

   return ( (long) ( n * 1000) );
}

