/* weightwin.cpp - Morse Code Dojo weight display / edit window
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include "weightwin.h"

#include <qcombobox.h>
#include <qpushbutton.h>
#include "graph.h"
#include "mrand.h"
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <stdio.h>
#include <stdlib.h>

/* 
 *  Constructs a WeightWin which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
WeightWin::WeightWin( QWidget* parent,  const char* name, bool modal, WFlags fl )
   : QDialog( parent, name, modal, fl )
{
   if ( !name )
      setName( "WeightWin" );
   resize( 396, 311 ); 
   setCaption( tr( "Character Weighting" ) );
   WeightWinLayout = new QVBoxLayout( this ); 
   WeightWinLayout->setSpacing( 6 );
   WeightWinLayout->setMargin( 11 );

   weight_g = new Graph( this, "weight_g" );
   WeightWinLayout->addWidget( weight_g );

   Layout1 = new QHBoxLayout; 
   Layout1->setSpacing( 6 );
   Layout1->setMargin( 0 );

   weight_v = new QComboBox( FALSE, this, "weight_v" );
   weight_v->insertItem( tr( "Random" ),  MRRandom );
   weight_v->insertItem( tr( "History" ), MRHistory );
   weight_v->insertItem( tr( "Letters" ), MRLetters );
   weight_v->insertItem( tr( "Numbers" ), MRNumbers );
   weight_v->insertItem( tr( "Text" ),    MRText );
   weight_v->insertItem( tr( "Student" ), MRStudent );
   Layout1->addWidget( weight_v );
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout1->addItem( spacer );

   ok_b = new QPushButton( this, "ok_b" );
   ok_b->setText( tr( "Ok" ) );
   ok_b->setDefault( TRUE );
   Layout1->addWidget( ok_b );

   PushButton1_2 = new QPushButton( this, "PushButton1_2" );
   PushButton1_2->setText( tr( "Cancel" ) );
   Layout1->addWidget( PushButton1_2 );
   WeightWinLayout->addLayout( Layout1 );

   // signals and slots connections
   connect( ok_b, SIGNAL( clicked() ), this, SLOT( accept() ) );
   connect( ok_b, SIGNAL( clicked() ), this, SLOT( close() ) );
   connect( ok_b, SIGNAL( clicked() ), this, SLOT( okPressed() ) );
   connect( PushButton1_2, SIGNAL( clicked() ), this, SLOT( close() ) );
   connect( PushButton1_2, SIGNAL( clicked() ), this, SLOT( reject() ) );
   connect( weight_v, SIGNAL( activated(const QString&) ), this, SLOT( weightChanged(const QString&) ) );
   connect( weight_g, SIGNAL( clickedAt(int, int) ), this, SLOT( newPoint(int, int) ) );

   weight_g->setScale( 25 );
   m_mRand = MRand::getInstance();
   updateGraph();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
WeightWin::~WeightWin()
{
   // no need to delete child widgets, Qt does it all for us
}

void WeightWin::weightChanged( const QString& s )
{
   if ( s == QString("Random") )
   {
      m_mRand->setWeight( MRRandom );
   }
   else if ( s == QString("History") )
   {
      m_mRand->setWeight( MRHistory );
   }
   else if ( s == QString("Student") )
   {
      m_mRand->setWeight( m_student );
   }
   else if ( s == QString("Text") )
   {
      m_mRand->setWeight( MRText );
   }
   else if ( s == QString("Letters") )
   {
      m_mRand->setWeight( MRLetters );
   }
   else if ( s == QString("Numbers") )
   {
      m_mRand->setWeight( MRNumbers );
   }
   else
   {
      fprintf( stderr, "Unknown selection %s\n", s.latin1() );
   }

   updateGraph();

   if ( s == QString("Student") )
      weight_g->setClickable( true );
}

void WeightWin::updateGraph( void )
{
   weight_g->reset();
   weight_g->setScale( 25 );

   for ( int t = 0; t < 43; t++ )
   {
      weight_g->addPoint(t, m_mRand->getAbsWeight(t) );
   }

   weight_g->showGraph();
   return;
}

void WeightWin::closeEvent(QCloseEvent *e)
{
   emit closed();
   QDialog::closeEvent(e);
}

void WeightWin::newPoint( int x, int y )
{
   weight_g->addPoint(x, y);
   m_student.setWeight(x, y);
//   m_mRand->setAbsWeight(x, y);
}

void WeightWin::show(void)
{
   m_student = WStudent;
//   m_mRand->setWeight( );
   weight_v->setCurrentItem( m_mRand->currentWeight() );
   updateGraph();

   if ( m_mRand->currentWeight() == MRStudent )
      weight_g->setClickable( true );
   else
      weight_g->setClickable( false );


   QDialog::show();
}

void WeightWin::okPressed(void)
{
   MRWeight w = (MRWeight) weight_v->currentItem();
   if ( w == MRStudent)
   {
      printf("Setting student\n");
      WStudent = m_student;
   }

   m_mRand->setWeight(w);
}
